% Figure 7 in "The Limited Power of Monetary Policy in a Pandemic",
% Antoine Lepetit and Cristina Fuentes-Albero, May 2022.

%% Preparation

clear all
clc

mp_pandemic_zlb_cont;

global M_ options_ oo_;
Tsim = 1000;

options_.periods = Tsim;
options_.simul.maxit = 200; 
options_.dynatol.f = 1e-10;
options_.dynatol.x = 1e-10;

maxlag=1;
maxlead=1;
endo_simul = repmat(oo_.endo_simul(:,1),[1,Tsim+maxlag+maxlead]);
endo_simul(:,end) = oo_.endo_simul(:,end);
oo_.endo_simul = endo_simul;

% Calibrate shocks

covid_19 =[0.005;zeros(Tsim,1)];
mp_shock = [zeros(Tsim+1,1)];

shocks = [mp_shock,covid_19];

eps_t = shocks(1:2,:);
Tshock = size(eps_t,1);
nshock = size(eps_t,2);

%% Simulate baseline

Xn_baseline = endo_simul;

for h=1:Tshock
    
    display(sprintf('Now on time %d', h));
    
    options_.periods = Tsim-h+1;
    
    oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[zeros(Tsim-h+1,1),zeros(Tsim-h+1,1)]];
    perfect_foresight_solver;
    
    Xn_baseline(:,h+1:end) = real(oo_.endo_simul(:,2:end));
    oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
end

save('Xn_baseline','Xn_baseline');
clear all;

%% Simulate economy with optimal policies

optimal_lockdown_and_monetary_policy;

global M_ options_ oo_;
Tsim = 1000;

options_.periods = Tsim;
options_.simul.maxit = 200; 
options_.dynatol.f = 1e-10;
options_.dynatol.x = 1e-10;

maxlag=1;
maxlead=1;
endo_simul = repmat(oo_.endo_simul(:,1),[1,Tsim+maxlag+maxlead]);
endo_simul(:,end) = oo_.endo_simul(:,end);
oo_.endo_simul = endo_simul;



% Calibrate shocks

covid_19 =[0.005;zeros(Tsim,1)];

shocks = [covid_19];

eps_t = shocks(1:2,:);
Tshock = size(eps_t,1);
nshock = size(eps_t,2);

Xn_optimal = endo_simul;

for h=1:Tshock
    
    display(sprintf('Now on time %d', h));
    
    options_.periods = Tsim-h+1;  
    oo_.exo_simul = [zeros(1,nshock);eps_t(h,:);[covid_19(h+1:end)]];
    perfect_foresight_solver;
    
    Xn_optimal(:,h+1:end) = real(oo_.endo_simul(:,2:end));
    oo_.endo_simul = real(oo_.endo_simul(:,2:end));
    
end

clear oo_.endo_simul oo_.exo_simul shocks eps_t Tshock nshock;

mu_optimal = mu;
save('mu_optimal','mu_optimal');

%% Figures
Xn_baseline = load('Xn_baseline.mat');
figure(1);
plot(Xn_baseline.Xn_baseline(3,3:500),'LineWidth',3);
hold on
plot(Xn_optimal(3,3:500),'--','LineWidth',3);
%title('Infected','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Fraction of the population','Fontsize',18)
saveas(gcf,'figures/fig7_infected.eps','epsc')

figure(2);
plot(Xn_baseline.Xn_baseline(2,3:500),'LineWidth',3);
hold on
plot(Xn_optimal(2,3:500),'--','LineWidth',3);
%title('Susceptible','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Fraction of the population','Fontsize',18)
saveas(gcf,'figures/fig7_susceptible.eps','epsc')

figure(3);
plot((Xn_baseline.Xn_baseline(12,3:500)-1)*100,'LineWidth',3);
hold on
plot((Xn_optimal(11,3:500)-1)*100,'--','LineWidth',3);
%title('Output','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Percentage deviation','Fontsize',18)
saveas(gcf,'figures/fig7_output.eps','epsc')

figure(4);
plot((Xn_baseline.Xn_baseline(14,3:500)-1)*100,'LineWidth',3);
hold on
plot((Xn_optimal(13,3:500)-1)*100,'--','LineWidth',3);
%title('Consumption','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Percentage deviation','Fontsize',18)
saveas(gcf,'figures/fig7_consumption.eps','epsc')

figure(5);
plot((Xn_baseline.Xn_baseline(9,3:500)-1)*100,'LineWidth',3);
hold on
plot((Xn_optimal(9,3:500)-1)*100,'--','LineWidth',3);
%title('Inflation','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Percentage points (weekly)','Fontsize',18)
saveas(gcf,'figures/fig7_inflation.eps','epsc')

figure(6);
plot(Xn_baseline.Xn_baseline(10,3:500)*100,'LineWidth',3);
hold on
plot(Xn_optimal(10,3:500)*100,'--','LineWidth',3);
%title('Policy rate','Fontsize',20)
xlabel('Weeks','Fontsize',18)
ylabel('Percentage points (weekly)','Fontsize',18)
saveas(gcf,'figures/fig7_ffr.eps','epsc')


figure(7);
plot(Xn_baseline.Xn_baseline(5,3:500),'LineWidth',3);
hold on
plot(Xn_optimal(5,3:500),'--','LineWidth',3);
%title('Cumulative death','Fontsize',20)
xlabel('Weeks','FontSize',18);
ylabel('Fraction of population','FontSize',18);
saveas(gcf,'figures/fig7_death.eps','epsc')

figure(8);
plot(Xn_baseline.Xn_baseline(24,3:500),'LineWidth',3);
hold on
plot(Xn_optimal(23,3:500),'--','LineWidth',3);
%title('Contaiment rate','Fontsize',20)
xlabel('Weeks','Fontsize',18)
saveas(gcf,'figures/fig7_contaiment.eps','epsc')