/* 
New Keynesian model with SIR-type epidemic dynamics & stochastic vaccine arrival
The Limited Power of Monetary Policy in a Pandemic, Antoine Lepetit and Cristina Fuentes-Albero, May 2022
Version where lockdown and monetary policies are conducted optimally
*/

var t s i r d R0 tau w infl ffr y lambda c c_s c_i c_r n_s n_i n_r V_s V_i V_r mu
    lambda_1 lambda_2 lambda_3 lambda_4 lambda_5 lambda_6 lambda_7 lambda_8 lambda_9 lambda_10 lambda_11 lambda_12 lambda_13 lambda_14 lambda_15 lambda_16;

varexo covid_19; 

parameters pi_s1 pi_s2 pi_s3 pi_r pi_d R0_ss share delta_v
           beta teta phip delta_infl delta_gap xi varphi sigma phi A infl_target flow_value_life subsidy
           ffr_init n_s_init n_i_init n_r_init w_init c_s_init c_i_init c_r_init V_s_init V_i_init V_r_init y_init lambda_init mu_init
           ffr_end n_s_end n_i_end n_r_end w_end c_s_end c_i_end c_r_end V_s_end V_i_end V_r_end y_end lambda_end mu_end
           s_init i_init r_init s_end i_end r_end d_end
           lambda_15_init lambda_5_init lambda_7_init lambda_8_init lambda_10_init lambda_14_init lambda_11_init lambda_12_init lambda_13_init
           lambda_1_end lambda_2_end lambda_3_end lambda_4_end lambda_5_end lambda_6_end lambda_7_end lambda_8_end lambda_9_end lambda_10_end lambda_11_end lambda_12_end lambda_13_end lambda_14_end lambda_15_end lambda_16_end;

// SIR PARAMETERS // 

pi_d = 7*0.005/15;
pi_r = 7/15 - pi_d;
A = 1;
n_s_init = 1;
share = 1/4;
R0_ss = 1.45;

pi_s3 = R0_ss*(pi_r+pi_d)/(1 + 2*(share/(1-share))*(1+share/(1-share))/(1 - (share/(1-share))^2));

delta_v = 1/52;

// NK PARAMETERS //

beta = 0.9996;
teta = 6;
phip = 40926;
delta_infl = 1.5;
delta_gap = 0;
varphi = 0.5;
sigma = 2;
phi = 0.8;
infl_target = 1.02^(1/52);
subsidy = 1/(teta-1);

// INITIAL STEADY STATE //


s_init = 1;
i_init = 0;
r_init = 0;
ffr_init = infl_target/beta - 1;
w_init = 1;
xi = 1/((n_s_init^(1/varphi+sigma))*(A^sigma));
c_s_init = A*n_s_init;
c_i_init = c_s_init;
c_r_init = c_s_init;
n_i_init = (phi/(xi*(c_i_init^sigma)))^varphi;
n_r_init = n_s_init;
y_init = A*n_s_init;
mu_init = 0;
lambda_init = (c_s_init^(-sigma))/(1+mu_init);
flow_value_life = (1-beta)*10310 - (c_s_init^(1-sigma))/(1-sigma) + xi*(n_s_init^(1+1/varphi))/(1+1/varphi); //10310
V_r_init = ((c_r_init^(1-sigma))/(1-sigma) - xi*(n_r_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_init*(w_init*n_s_init - (1+mu_init)*c_s_init))/(1-beta);
V_i_init = ((c_i_init^(1-sigma))/(1-sigma) - xi*(n_i_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_init*(w_init*phi*n_i_init - (1+mu_init)*c_i_init) + beta*pi_r*V_r_init)/(1-beta*(1 - pi_r - pi_d));
V_s_init = ((c_s_init^(1-sigma))/(1-sigma) - xi*(n_s_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_init*(w_init*n_r_init - (1+mu_init)*c_r_init) + beta*delta_v*V_r_init)/(1-beta*(1-delta_v));

pi_s1 = (share/(1-share))*(1+share/(1-share))*pi_s3/((1 - (share/(1-share))^2)*(c_s_init*c_i_init));
pi_s2 = (share/(1-share))*(1+share/(1-share))*pi_s3/((1 - (share/(1-share))^2)*(n_s_init*n_i_init));

lambda_15_init = -(c_s_init^(-sigma));
lambda_5_init = s_init*(c_s_init^(-sigma) + lambda_15_init)*(c_s_init^(1+sigma))/sigma;
lambda_7_init = r_init*(c_r_init^(-sigma) + lambda_15_init)*(c_r_init^(1+sigma))/sigma;
lambda_8_init = -s_init*(xi*(n_s_init^(1/varphi)) + lambda_15_init)*(varphi/xi)*(n_s_init^(1 - 1/varphi));
lambda_10_init = -r_init*(xi*(n_r_init^(1/varphi)) + lambda_15_init)*(varphi/xi)*(n_r_init^(1 - 1/varphi));
lambda_14_init = -(c_s_init^(-sigma))*(lambda_8_init + lambda_10_init)/teta;
lambda_11_init = 0;
lambda_12_init = 0;
lambda_13_init = 0;
lambda_16_init = -lambda_15_init;
lambda_3_init = beta*((c_r_init^(1-sigma))/(1-sigma) - xi*(n_r_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_15_init*(c_r_init - n_r_init))/(1-beta); 
lambda_1_init = beta*((c_s_init^(1-sigma))/(1-sigma) - xi*(n_s_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_3_init*delta_v + lambda_15_init*(c_s_init-n_s_init))/(1-beta*(1-delta_v));
lambda_2_init = beta*((c_i_init^(1-sigma))/(1-sigma) - xi*(n_i_init^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_3_init*pi_r - (lambda_1_init*(1-delta_v)*s_init + lambda_3_init*delta_v*s_init)*(pi_s1*c_s_init*c_i_init + pi_s2*n_s_init*n_i_init + pi_s3) + (lambda_5_init*pi_s1*c_i_init + lambda_8_init*pi_s2*n_i_init)*beta*(V_i_init - (1-delta_v)*V_s_init - delta_v*V_r_init) + lambda_15_init*(c_i_init - phi*n_i_init))/(1 - beta*(1 - pi_r - pi_d) - s_init*beta*(pi_s1*c_s_init*c_i_init + pi_s2*n_s_init*n_i_init + pi_s3));
lambda_4_init = -lambda_1_init*(1-delta_v)*s_init + lambda_2_init*s_init - lambda_3_init*delta_v*s_init;
lambda_6_init = 0;
lambda_9_init = 0;


// FINAL STEADY STATE //

// Enter a guess for number of deaths (as a fraction of the population), number of susceptibles & number of recovered
s_end = 0.65;
i_end = 0;
r_end = 1-s_end;
d_end = 0.0016;

w_end = w_init;
n_s_end = n_s_init;
n_i_end = n_i_init;
n_r_end = n_r_init;
y_end = A*(s_end*n_s_end + r_end*n_r_end);
c_s_end = c_s_init;
c_i_end = c_i_init;
c_r_end = c_r_init;
ffr_end = ffr_init;
mu_end = 0;
lambda_end = (c_s_end^(-sigma))/(1+mu_end);
V_r_end = ((c_r_end^(1-sigma))/(1-sigma) - xi*(n_r_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_end*(w_end*n_s_end - (1+mu_end)*c_s_end))/(1-beta);
V_i_end = ((c_i_end^(1-sigma))/(1-sigma) - xi*(n_i_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_end*(w_end*phi*n_i_end - (1+mu_end)*c_i_end) + beta*pi_r*V_r_end)/(1-beta*(1 - pi_r - pi_d));
V_s_end = ((c_s_end^(1-sigma))/(1-sigma) - xi*(n_s_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_end*(w_end*n_r_end - (1+mu_end)*c_r_end) + beta*delta_v*V_r_end)/(1-beta*(1-delta_v));

lambda_15_end = -(c_s_end^(-sigma));
lambda_5_end = s_end*(c_s_end^(-sigma) + lambda_15_end)*(c_s_end^(1+sigma))/sigma;
lambda_7_end = r_end*(c_r_end^(-sigma) + lambda_15_end)*(c_r_end^(1+sigma))/sigma;
lambda_8_end = -s_end*(xi*(n_s_end^(1/varphi)) + lambda_15_end)*(varphi/xi)*(n_s_end^(1 - 1/varphi));
lambda_10_end = -r_end*(xi*(n_r_end^(1/varphi)) + lambda_15_end)*(varphi/xi)*(n_r_end^(1 - 1/varphi));
lambda_14_end = -(c_s_end^(-sigma))*(lambda_8_end + lambda_10_end)/teta;
lambda_11_end = 0;
lambda_12_end = 0;
lambda_13_end = 0;
lambda_16_end = -lambda_15_end;
lambda_3_end = beta*((c_r_end^(1-sigma))/(1-sigma) - xi*(n_r_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_15_end*(c_r_end - n_r_end))/(1-beta); 
lambda_1_end = beta*((c_s_end^(1-sigma))/(1-sigma) - xi*(n_s_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_3_end*delta_v + lambda_15_end*(c_s_end-n_s_end))/(1-beta*(1-delta_v));
lambda_2_end = beta*((c_i_end^(1-sigma))/(1-sigma) - xi*(n_i_end^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_3_end*pi_r - (lambda_1_end*(1-delta_v)*s_end + lambda_3_end*delta_v*s_end)*(pi_s1*c_s_end*c_i_end + pi_s2*n_s_end*n_i_end + pi_s3) + (lambda_5_end*pi_s1*c_i_end + lambda_8_end*pi_s2*n_i_end)*beta*(V_i_end - (1-delta_v)*V_s_end - delta_v*V_r_end) + lambda_15_end*(c_i_end - phi*n_i_end))/(1 - beta*(1 - pi_r - pi_d) - s_end*beta*(pi_s1*c_s_end*c_i_end + pi_s2*n_s_end*n_i_end + pi_s3));
lambda_4_end = -lambda_1_end*(1-delta_v)*s_end + lambda_2_end*s_end - lambda_3_end*delta_v*s_end;
lambda_6_end = 0;
lambda_9_end = 0;


model;

//                           //
// DECENTRALIZED EQUILIBRIUM //
//                           //

// SIR BLOCK //

[name = 'Number of newly-infected']
t = pi_s1*s(-1)*c_s*i(-1)*c_i + pi_s2*s(-1)*n_s*i(-1)*n_i + pi_s3*s(-1)*i(-1);

[name = 'Law of motion of susceptibles']
s = s(-1) - t - covid_19;

[name = 'Law of motion of infected']
i = (1 - pi_r - pi_d)*i(-1) + t + covid_19;
   
[name = 'Law of motion of recovered']
r = r(-1) + pi_r*i(-1);

[name = 'Law of motion of deaths']
d = d(-1) + pi_d*i(-1);

[name = 'Probability of getting infected']
tau = t/s(-1);

[name = 'R0']
R0 = (pi_s1*c_s*c_i + pi_s2*n_s*n_i + pi_s3)/(pi_r + pi_d);

// NK BLOCK //

// Aggregate equations

[name = 'Price Phillips curve']
(1 - teta)*(1 + subsidy) + teta*w/A - phip*infl*(infl-infl_target) + beta*phip*infl(+1)*(infl(+1)-infl_target)*y(+1)/y = 0;

[name = 'Resource constraint']
y*(1 - (phip/2)*((infl-infl_target)^2)) = s(-1)*c_s + i(-1)*c_i + r(-1)*c_r;

[name = 'Production function']
y = A*(s(-1)*n_s + i(-1)*phi*n_i + r(-1)*n_r);

[name = 'Aggregate consumption']
c = s(-1)*c_s + i(-1)*c_i + r(-1)*c_r;

//[name = 'Monetary policy']
//1+ffr = (1+ffr_init)*((infl/infl_target)^delta_infl)*((y/y_end)^delta_gap);

// Households

[name = 'Marginal utility of consumption - susceptibles']                      
lambda*(1+mu) = c_s^(-sigma) +  pi_s1*i(-1)*c_i*beta*(V_i(+1) - (1-delta_v)*V_s(+1) - delta_v*V_r(+1));

[name = 'Marginal utility of consumption - infected']                      
lambda*(1+mu) = c_i^(-sigma);

[name = 'Marginal utility of consumption - recovered']                      
lambda*(1+mu) = c_r^(-sigma);

[name = 'Labor supply - susceptibles']                      
lambda*w = xi*(n_s^(1/varphi)) - pi_s2*i(-1)*n_i*beta*(V_i(+1) - (1-delta_v)*V_s(+1) - delta_v*V_r(+1));

[name = 'Labor supply - infected']                      
lambda*w*phi = xi*(n_i^(1/varphi));

[name = 'Labor supply - recovered']                      
lambda*w = xi*(n_r^(1/varphi));

[name = 'Euler equation']
1+ffr = (lambda/lambda(+1))*(infl(+1)/beta) ;

[name = 'Lifetime utility of a susceptible']
V_s = (c_s^(1-sigma))/(1-sigma) - xi*(n_s^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda*(w*n_s - (1+mu)*c_s) + (1-tau)*(1-delta_v)*beta*V_s(+1) + tau*beta*V_i(+1) + (1-tau)*delta_v*beta*V_r(+1);

[name = 'Lifetime utility of an infected']
V_i = (c_i^(1-sigma))/(1-sigma) - xi*(n_i^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda*(w*phi*n_i - (1+mu)*c_i) + (1-pi_r-pi_d)*beta*V_i(+1) + pi_r*beta*V_r(+1);

[name = 'Lifetime utility of a recovered']
V_r = (c_r^(1-sigma))/(1-sigma) - xi*(n_r^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda*(w*n_r - (1+mu)*c_r) + beta*V_r(+1);

//             //
// RAMSEY FOCs //
//             //

[name = 'FOC wrt S']
lambda_1 = beta*((c_s(+1)^(1-sigma))/(1-sigma) - xi*(n_s(+1)^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_1(+1)*(1-delta_v)*(1-tau(+1)) + lambda_2(+1)*tau(+1) + lambda_3(+1)*delta_v*(1-tau(+1)) + lambda_15(+1)*c_s(+1) + lambda_16(+1)*n_s(+1));

[name = 'FOC wrt I']
lambda_2 = beta*((c_i(+1)^(1-sigma))/(1-sigma) - xi*(n_i(+1)^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_2(+1)*(1-pi_r-pi_d) + lambda_3(+1)*pi_r + lambda_4(+1)*(pi_s1*c_s(+1)*c_i(+1) + pi_s2*n_s(+1)*n_i(+1) + pi_s3)
           + (lambda_5(+1)*pi_s1*c_i(+1) + lambda_8(+1)*pi_s2*n_i(+1))*beta*(V_i(+2) - (1-delta_v)*V_s(+2) - delta_v*V_r(+2)) + lambda_15(+1)*c_i(+1) + lambda_16(+1)*phi*n_i(+1));

[name = 'FOC wrt R']
lambda_3 = beta*((c_r(+1)^(1-sigma))/(1-sigma) - xi*(n_r(+1)^(1+1/varphi))/(1+1/varphi) + flow_value_life + lambda_3(+1) + lambda_15(+1)*c_r(+1) + lambda_16(+1)*n_r(+1)); 

[name = 'FOC wrt tau']
-lambda_1*(1-delta_v)*s(-1) + lambda_2*s(-1) - lambda_3*delta_v*s(-1) - lambda_4 + lambda_11*beta*(V_i(+1) - (1-delta_v)*V_s(+1) - delta_v*V_r(+1)) = 0;

[name = 'FOC wrt C_s']
s(-1)*(c_s^(-sigma)) + lambda_4*pi_s1*i(-1)*c_i - lambda_5*sigma*(c_s^(-sigma-1)) + lambda_11*(c_s^(-sigma) - lambda*(1+mu)) + lambda_15*s(-1) = 0;

[name = 'FOC wrt C_i']
i(-1)*(c_i^(-sigma)) + lambda_4*pi_s1*c_s*i(-1) + lambda_5*pi_s1*i(-1)*beta*(V_i(+1)-(1-delta_v)*V_s(+1)-delta_v*V_r(+1)) - lambda_6*sigma*(c_i^(-sigma-1)) + lambda_12*(c_i^(-sigma) - lambda*(1+mu)) + lambda_15*i(-1) = 0;

[name = 'FOC wrt C_r']
r(-1)*(c_r^(-sigma)) - lambda_7*sigma*(c_r^(-sigma-1)) + lambda_13*(c_r^(-sigma) - lambda*(1+mu)) + lambda_15*r(-1) = 0;

[name = 'FOC wrt N_s']
-s(-1)*xi*(n_s^(1/varphi)) + lambda_4*pi_s2*i(-1)*n_i - lambda_8*(xi/varphi)*(n_s^(1/varphi-1)) + lambda_11*(-xi*(n_s^(1/varphi)) + lambda*w) + lambda_16*s(-1) = 0;

[name = 'FOC wrt N_i']
-i(-1)*xi*(n_i^(1/varphi)) + lambda_4*pi_s2*n_s*i(-1) + lambda_8*pi_s2*i(-1)*beta*(V_i(+1) - (1-delta_v)*V_s(+1) - delta_v*V_r(+1)) - lambda_9*(xi/varphi)*(n_i^(1/varphi-1)) + lambda_12*(-xi*(n_i^(1/varphi)) + lambda*w*phi) + lambda_16*phi*i(-1) = 0;

[name = 'FOC wrt N_r']
-r(-1)*xi*(n_r^(1/varphi)) - lambda_10*(xi/varphi)*(n_r^(1/varphi-1)) + lambda_13*(-xi*(n_r^(1/varphi)) + lambda*w) + lambda_16*r(-1) = 0;

[name = 'FOC wrt V_s']
-lambda_11 - lambda_5(-1)*pi_s1*i(-2)*c_i(-1)*(1-delta_v) - lambda_8(-1)*pi_s2*i(-2)*n_i(-1)*(1-delta_v) + lambda_11(-1)*(1-tau(-1))*(1-delta_v) = 0;

[name = 'FOC wrt V_i']
-lambda_12 + lambda_5(-1)*pi_s1*i(-2)*c_i(-1) + lambda_8(-1)*pi_s2*i(-2)*n_i(-1) + lambda_11(-1)*tau(-1) + lambda_12(-1)*(1-pi_r-pi_d);

[name = 'FOC wrt V_r']
-lambda_13 - lambda_5(-1)*pi_s1*i(-2)*c_i(-1)*delta_v - lambda_8(-1)*pi_s2*i(-2)*n_i(-1)*delta_v + lambda_11(-1)*(1-tau(-1))*delta_v + lambda_12(-1)*pi_r + lambda_13(-1) = 0;

[name = 'FOC wrt lambda']
-(1+mu)*(lambda_5 + lambda_6 + lambda_7) + w*(lambda_8 + phi*lambda_9 + lambda_10) + lambda_11*(w*n_s - (1+mu)*c_s) + lambda_12*(w*phi*n_i - (1+mu)*c_i) + lambda_13*(w*n_r - (1+mu)*c_r) = 0;

[name = 'FOC wrt w']
lambda*(lambda_8 + phi*lambda_9 + lambda_10 + n_s*lambda_11 + phi*n_i*lambda_12 + n_r*lambda_13) + lambda_14*teta = 0;

[name = 'FOC wrt infl']
-lambda_14*phip*(2*infl - infl_target) + lambda_15*y*phip*(infl-infl_target) + lambda_14(-1)*phip*(2*infl - infl_target)*y/y(-1) = 0;

[name = 'FOC wrt y']
-lambda_14*beta*phip*infl(+1)*(infl(+1)-infl_target)*y(+1)/(y^2) + lambda_14(-1)*phip*infl*(infl-infl_target)/y(-1) - lambda_15*(1 - (phip/2)*((infl-infl_target)^2)) - lambda_16 = 0;

[name = 'FOC wrt mu']
-lambda*(lambda_5 + lambda_6 + lambda_7 + lambda_11*c_s + lambda_12*c_i + lambda_13*c_r) = 0;


end;

initval;
s = 1;
i = 0;
r = 0;
d = 0;
tau = 0;
R0 = (pi_s1*c_s_init*c_i_init + pi_s2*n_s_init*n_i_init + pi_s3)/(pi_r + pi_d);
w = w_init;
infl = infl_target;
ffr = ffr_init;
y = y_init;
lambda = lambda_init;
c = y_init;
c_s = c_s_init;
c_i = c_i_init; 
c_r = c_r_init;
n_s = n_s_init;
n_i = n_i_init;
n_r = n_r_init;
V_s = V_s_init; 
V_i = V_i_init; 
V_r = V_r_init;
mu = w_init - 1;
//lambda_1 = lambda_1_init;
//lambda_2 = lambda_2_init;
//lambda_3 = lambda_3_init;
//lambda_4 = lambda_4_init;
//lambda_5 = lambda_5_init;
//lambda_6 = lambda_6_init;
//lambda_7 = lambda_7_init;
//lambda_8 = lambda_8_init;
//lambda_9 = lambda_9_init;
//lambda_10 = lambda_10_init;
//lambda_11 = lambda_11_init;
//lambda_12 = lambda_12_init;
//lambda_13 = lambda_13_init;
//lambda_14 = lambda_14_init;
//lambda_15 = lambda_15_init;
//lambda_16 = lambda_16_init;
end;

resid;

endval;
t = 0;
s = s_end;
i = 0;
r = r_end;
d = d_end;
tau = 0;
R0 = (pi_s1*c_s_end*c_i_end + pi_s2*n_s_end*n_i_end + pi_s3)/(pi_r + pi_d);
w = w_end;
infl = infl_target;
ffr = ffr_end;
y = y_end;
lambda = lambda_end;
c = y_end;
c_s = c_s_end;
c_i = c_i_end; 
c_r = c_r_end;
n_s = n_s_end;
n_i = n_i_end;
n_r = n_r_end;
V_s = V_s_end; 
V_i = V_i_end; 
V_r = V_r_end;
mu = mu_end;
lambda_1 = lambda_1_end;
lambda_2 = lambda_2_end;
lambda_3 = lambda_3_end;
lambda_4 = lambda_4_end;
lambda_5 = lambda_5_end;
lambda_6 = lambda_6_end;
lambda_7 = lambda_7_end;
lambda_8 = lambda_8_end;
lambda_9 = lambda_9_end;
lambda_10 = lambda_10_end;
lambda_11 = lambda_11_end;
lambda_12 = lambda_12_end;
lambda_13 = lambda_13_end;
lambda_14 = lambda_14_end;
lambda_15 = lambda_15_end;
lambda_16 = lambda_16_end;
end;

resid;

check;

shocks;
var covid_19;
periods 1;
values 0.005;
end;

simul(periods=1000);